use crate::ems::pcs::pcs::Pcs;
use crate::ems::Service;
use crate::internal::utils;
use std::collections::HashMap;
use std::sync::Arc;
use tokio::sync;
use tokio::sync::Mutex;

type ServiceChannels = Arc<HashMap<String, Mutex<sync::mpsc::Receiver<String>>>>;

/// services 是所有实现了Service接口的设备集合
/// tx 是EMU向其他设备广播消息的发送器
/// service_channels 是所有设备向EMU发送消息的通道集合
pub struct Emu {
    pub services: Vec<Arc<Mutex<dyn Service>>>,
    pub tx: sync::broadcast::Sender<String>,
    pub service_channels: ServiceChannels,
}
impl Emu {
    pub async fn new() -> Self {
        //初始化日志
        utils::log::init_log("inpower_iot_mgc_rs::ems::emu::*", "logs/emu.log").await;
        let (tx, _) = sync::broadcast::channel::<String>(8);
        let mut services: Vec<Arc<Mutex<dyn Service>>> = Vec::new();
        let mut service_channels = HashMap::new();
        log::info!("EMU初始化完成");
        //↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓ 创建PCS
        let (pcs_tx, pcs_rx) = sync::mpsc::channel::<String>(8);
        let pcs = Arc::new(Mutex::new(Pcs::new(pcs_tx, tx.subscribe()).await));
        services.push(pcs);
        service_channels.insert(1.to_string(), Mutex::new(pcs_rx));
        //↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑
        Emu {
            services,
            service_channels: Arc::new(service_channels),
            tx,
        }
    }

    pub async fn run(self: Arc<Self>) {
        let mut handles = Vec::new();
        for service in self.services.iter() {
            let service_clone = service.clone();
            let handle = tokio::spawn(async move {
                service_clone.lock().await.serve().await;
            });
            handles.push(handle);
        }
        for handler in handles {
            match handler.await {
                Ok(_) => {}
                Err(e) => {
                    log::error!("内部错误: {}", e.to_string());
                    continue;
                }
            }
        }
    }
}
